@@ -20,17 +20,25 @@ import contextlib
 import datetime
 import inspect
 import io
-import random as rnd_module
 import sys
 import time
-import traceback
 import types
 import warnings
 import zlib
 from collections import defaultdict
 from io import StringIO
 from random import Random
-from typing import Any, BinaryIO, Callable, Hashable, List, Optional, TypeVar, Union
+from typing import (
+    Any,
+    BinaryIO,
+    Callable,
+    Coroutine,
+    Hashable,
+    List,
+    Optional,
+    TypeVar,
+    Union,
+)
 from unittest import TestCase
 
 import attr
@@ -72,6 +80,7 @@ from hypothesis.internal.conjecture.shrinker import sort_key
 from hypothesis.internal.entropy import deterministic_PRNG
 from hypothesis.internal.escalation import (
     escalate_hypothesis_internal_error,
+    format_exception,
     get_interesting_origin,
     get_trimmed_traceback,
 )
@@ -110,6 +119,7 @@ TestFunc = TypeVar("TestFunc", bound=Callable)
 
 running_under_pytest = False
 global_force_seed = None
+_hypothesis_global_random = None
 
 
 @attr.s()
@@ -196,15 +206,17 @@ def decode_failure(blob):
     try:
         buffer = base64.b64decode(blob)
     except Exception:
-        raise InvalidArgument(f"Invalid base64 encoded string: {blob!r}")
+        raise InvalidArgument(f"Invalid base64 encoded string: {blob!r}") from None
     prefix = buffer[:1]
     if prefix == b"\0":
         return buffer[1:]
     elif prefix == b"\1":
         try:
             return zlib.decompress(buffer[1:])
-        except zlib.error:
-            raise InvalidArgument(f"Invalid zlib compression for blob {blob!r}")
+        except zlib.error as err:
+            raise InvalidArgument(
+                f"Invalid zlib compression for blob {blob!r}"
+            ) from err
     else:
         raise InvalidArgument(
             f"Could not decode blob {blob!r}: Invalid start byte {prefix!r}"
@@ -214,7 +226,7 @@ def decode_failure(blob):
 class WithRunner(MappedSearchStrategy):
     def __init__(self, base, runner):
         assert runner is not None
-        MappedSearchStrategy.__init__(self, base)
+        super().__init__(base)
         self.runner = runner
 
     def do_draw(self, data):
@@ -407,7 +419,10 @@ def get_random_for_wrapped_test(test, wrapped_test):
     elif global_force_seed is not None:
         return Random(global_force_seed)
     else:
-        seed = rnd_module.getrandbits(128)
+        global _hypothesis_global_random
+        if _hypothesis_global_random is None:
+            _hypothesis_global_random = Random()
+        seed = _hypothesis_global_random.getrandbits(128)
         wrapped_test._hypothesis_internal_use_generated_seed = seed
         return Random(seed)
 
@@ -466,11 +481,11 @@ def skip_exceptions_to_reraise():
     # and more importantly it avoids possible side-effects :-)
     if "unittest" in sys.modules:
         exceptions.add(sys.modules["unittest"].SkipTest)
-    if "unittest2" in sys.modules:  # pragma: no cover
+    if "unittest2" in sys.modules:
         exceptions.add(sys.modules["unittest2"].SkipTest)
-    if "nose" in sys.modules:  # pragma: no cover
+    if "nose" in sys.modules:
         exceptions.add(sys.modules["nose"].SkipTest)
-    if "_pytest" in sys.modules:  # pragma: no branch
+    if "_pytest" in sys.modules:
         exceptions.add(sys.modules["_pytest"].outcomes.Skipped)
     return tuple(sorted(exceptions, key=str))
 
@@ -481,8 +496,12 @@ def failure_exceptions_to_catch():
     This is intended to cover most common test runners; if you would
     like another to be added please open an issue or pull request.
     """
-    exceptions = [Exception]
-    if "_pytest" in sys.modules:  # pragma: no branch
+    # While SystemExit and GeneratorExit are instances of BaseException, we also
+    # expect them to be deterministic - unlike KeyboardInterrupt - and so we treat
+    # them as standard exceptions, check for flakiness, etc.
+    # See https://github.com/HypothesisWorks/hypothesis/issues/2223 for details.
+    exceptions = [Exception, SystemExit, GeneratorExit]
+    if "_pytest" in sys.modules:
         exceptions.append(sys.modules["_pytest"].outcomes.Failed)
     return tuple(exceptions)
 
@@ -631,14 +650,12 @@ class StateForActualGivenExecution:
                 and self.__test_runtime is not None
             ):
                 report(
-                    (
-                        "Unreliable test timings! On an initial run, this "
-                        "test took %.2fms, which exceeded the deadline of "
-                        "%.2fms, but on a subsequent run it took %.2f ms, "
-                        "which did not. If you expect this sort of "
-                        "variability in your test timings, consider turning "
-                        "deadlines off for this test by setting deadline=None."
-                    )
+                    "Unreliable test timings! On an initial run, this "
+                    "test took %.2fms, which exceeded the deadline of "
+                    "%.2fms, but on a subsequent run it took %.2f ms, "
+                    "which did not. If you expect this sort of "
+                    "variability in your test timings, consider turning "
+                    "deadlines off for this test by setting deadline=None."
                     % (
                         exception.runtime.total_seconds() * 1000,
                         self.settings.deadline.total_seconds() * 1000,
@@ -648,10 +665,8 @@ class StateForActualGivenExecution:
             else:
                 report("Failed to reproduce exception. Expected: \n" + traceback)
             self.__flaky(
-                (
-                    "Hypothesis %s(%s) produces unreliable results: Falsified"
-                    " on the first call but did not on a subsequent one"
-                )
+                "Hypothesis %s(%s) produces unreliable results: Falsified"
+                " on the first call but did not on a subsequent one"
                 % (test.__name__, text_repr[0])
             )
         return result
@@ -718,7 +733,7 @@ class StateForActualGivenExecution:
                 # This can happen if an error occurred in a finally
                 # block somewhere, suppressing our original StopTest.
                 # We raise a new one here to resume normal operation.
-                raise StopTest(data.testcounter)
+                raise StopTest(data.testcounter) from e
             else:
                 # The test failed by raising an exception, so we inform the
                 # engine that this test run was interesting. This is the normal
@@ -726,9 +741,7 @@ class StateForActualGivenExecution:
 
                 tb = get_trimmed_traceback()
                 info = data.extra_information
-                info.__expected_traceback = "".join(
-                    traceback.format_exception(type(e), e, tb)
-                )
+                info.__expected_traceback = format_exception(e, tb)
                 info.__expected_exception = e
                 verbose_report(info.__expected_traceback)
 
@@ -814,8 +827,8 @@ class StateForActualGivenExecution:
                         info.__expected_traceback,
                     ),
                 )
-            except (UnsatisfiedAssumption, StopTest):
-                report(traceback.format_exc())
+            except (UnsatisfiedAssumption, StopTest) as e:
+                report(format_exception(e, e.__traceback__))
                 self.__flaky(
                     "Unreliable assumption: An example which satisfied "
                     "assumptions on the first run now fails it."
@@ -833,7 +846,7 @@ class StateForActualGivenExecution:
                 # We are reporting multiple failures, so we need to manually
                 # print each exception's stack trace and information.
                 tb = get_trimmed_traceback()
-                report("".join(traceback.format_exception(type(e), e, tb)))
+                report(format_exception(e, tb))
 
             finally:  # pragma: no cover
                 # Mostly useful for ``find`` and ensuring that objects that
@@ -849,14 +862,11 @@ class StateForActualGivenExecution:
                 # branch to verify this and see it fail - and additionally the
                 # second branch still complains about lack of coverage even if
                 # you add a pragma: no cover to it!
-                # See https://bitbucket.org/ned/coveragepy/issues/623/
+                # See https://github.com/nedbat/coveragepy/issues/623
                 if self.settings.print_blob:
                     report(
-                        (
-                            "\nYou can reproduce this example by temporarily "
-                            "adding @reproduce_failure(%r, %r) as a decorator "
-                            "on your test case"
-                        )
+                        "\nYou can reproduce this example by temporarily adding "
+                        "@reproduce_failure(%r, %r) as a decorator on your test case"
                         % (__version__, encode_failure(falsifying_example.buffer))
                     )
             if self.__was_flaky:
@@ -948,7 +958,9 @@ class HypothesisHandle:
 def given(
     *_given_arguments: Union[SearchStrategy, InferType],
     **_given_kwargs: Union[SearchStrategy, InferType],
-) -> Callable[[Callable[..., None]], Callable[..., None]]:
+) -> Callable[
+    [Callable[..., Union[None, Coroutine[Any, Any, None]]]], Callable[..., None]
+]:
     """A decorator for turning a test function that accepts arguments into a
     randomized test.
 
@@ -1081,12 +1093,10 @@ def given(
                 expected_version, failure = reproduce_failure
                 if expected_version != __version__:
                     raise InvalidArgument(
-                        (
-                            "Attempting to reproduce a failure from a different "
-                            "version of Hypothesis. This failure is from %s, but "
-                            "you are currently running %r. Please change your "
-                            "Hypothesis version to a matching one."
-                        )
+                        "Attempting to reproduce a failure from a different "
+                        "version of Hypothesis. This failure is from %s, but "
+                        "you are currently running %r. Please change your "
+                        "Hypothesis version to a matching one."
                         % (expected_version, __version__)
                     )
                 try:
@@ -1104,13 +1114,13 @@ def given(
                         "The shape of the test data has changed in some way "
                         "from where this blob was defined. Are you sure "
                         "you're running the same test?"
-                    )
+                    ) from None
                 except UnsatisfiedAssumption:
                     raise DidNotReproduce(
                         "The test data failed to satisfy an assumption in the "
                         "test. Have you added it since this blob was "
                         "generated?"
-                    )
+                    ) from None
 
             # There was no @reproduce_failure, so start by running any explicit
             # examples from @example decorators.
@@ -1125,10 +1135,7 @@ def given(
                     for fragments, err in errors:
                         for f in fragments:
                             report(f)
-                        tb_lines = traceback.format_exception(
-                            type(err), err, err.__traceback__
-                        )
-                        report("".join(tb_lines))
+                        report(format_exception(err, err.__traceback__))
                     raise MultipleFailures(
                         f"Hypothesis found {len(errors)} failures in explicit examples."
                     )
