@@ -19,6 +19,7 @@ from typing import Any, Mapping, Optional, Sequence, Tuple, Union
 import numpy as np
 
 from hypothesis import strategies as st
+from hypothesis._settings import note_deprecation
 from hypothesis.errors import InvalidArgument
 from hypothesis.extra._array_helpers import (
     NDIM_MAX,
@@ -746,12 +747,13 @@ mutually_broadcastable_shapes.__doc__ = f"""
 
     **Use with Generalised Universal Function signatures**
 
-    A :np-ref:`universal function <ufuncs.html>` (or ufunc for short) is a function
+    A :doc:`universal function <numpy:reference/ufuncs>` (or ufunc for short) is a function
     that operates on ndarrays in an element-by-element fashion, supporting array
     broadcasting, type casting, and several other standard features.
-    A :np-ref:`generalised ufunc <c-api/generalized-ufuncs.html>` operates on
+    A :doc:`generalised ufunc <numpy:reference/c-api/generalized-ufuncs>` operates on
     sub-arrays rather than elements, based on the "signature" of the function.
-    Compare e.g. :obj:`numpy:numpy.add` (ufunc) to :obj:`numpy:numpy.matmul` (gufunc).
+    Compare e.g. :obj:`numpy.add() <numpy:numpy.add>` (ufunc) to
+    :obj:`numpy.matmul() <numpy:numpy.matmul>` (gufunc).
 
     To generate shapes for a gufunc, you can pass the ``signature`` argument instead of
     ``num_shapes``.  This must be a gufunc signature string; which you can write by
@@ -790,7 +792,7 @@ def basic_indices(
     allow_newaxis: bool = False,
     allow_ellipsis: bool = True,
 ) -> st.SearchStrategy[BasicIndex]:
-    """Return a strategy for :np-ref:`basic indexes <arrays.indexing.html>` of
+    """Return a strategy for :doc:`basic indexes <numpy:reference/arrays.indexing>` of
     arrays with the specified shape, which may include dimensions of size zero.
 
     It generates tuples containing some mix of integers, :obj:`python:slice`
@@ -803,10 +805,11 @@ def basic_indices(
       a valid index; for one-dimensional arrays use
       :func:`~hypothesis.strategies.slices` instead.
     * ``min_dims`` is the minimum dimensionality of the resulting array from use
-      of the generated index. When ``min_dims == 0``, indices for zero-dimensional
-      arrays are generated.
+      of the generated index. When ``min_dims == 0``, scalars and zero-dimensional
+      arrays are both allowed.
     * ``max_dims`` is the the maximum dimensionality of the resulting array,
-      defaulting to ``max(len(shape), min_dims) + 2``.
+      defaulting to ``len(shape) if not allow_newaxis else
+      max(len(shape), min_dims) + 2``.
     * ``allow_newaxis`` specifies whether ``None`` is allowed in the index.
     * ``allow_ellipsis`` specifies whether ``...`` is allowed in the index.
     """
@@ -815,23 +818,42 @@ def basic_indices(
     # cases if they're dealing in general indexers, and while it's fiddly we can
     # back-compatibly add them later (hence using kwonlyargs).
     check_type(tuple, shape, "shape")
+    check_argument(
+        all(isinstance(x, int) and x >= 0 for x in shape),
+        f"shape={shape!r}, but all dimensions must be non-negative integers.",
+    )
     check_type(bool, allow_ellipsis, "allow_ellipsis")
     check_type(bool, allow_newaxis, "allow_newaxis")
     check_type(int, min_dims, "min_dims")
+    if min_dims > len(shape) and not allow_newaxis:
+        note_deprecation(
+            f"min_dims={min_dims} is larger than len(shape)={len(shape)}, "
+            "but allow_newaxis=False makes it impossible for an indexing "
+            "operation to add dimensions.",
+            since="2021-09-15",
+            has_codemod=False,
+        )
     check_valid_dims(min_dims, "min_dims")
 
     if max_dims is None:
-        max_dims = min(max(len(shape), min_dims) + 2, NDIM_MAX)
-    check_type(int, max_dims, "max_dims")
+        if allow_newaxis:
+            max_dims = min(max(len(shape), min_dims) + 2, NDIM_MAX)
+        else:
+            max_dims = min(len(shape), NDIM_MAX)
+    else:
+        check_type(int, max_dims, "max_dims")
+        if max_dims > len(shape) and not allow_newaxis:
+            note_deprecation(
+                f"max_dims={max_dims} is larger than len(shape)={len(shape)}, "
+                "but allow_newaxis=False makes it impossible for an indexing "
+                "operation to add dimensions.",
+                since="2021-09-15",
+                has_codemod=False,
+            )
     check_valid_dims(max_dims, "max_dims")
 
     order_check("dims", 0, min_dims, max_dims)
 
-    if not all(isinstance(x, int) and x >= 0 for x in shape):
-        raise InvalidArgument(
-            f"shape={shape!r}, but all dimensions must be of integer size >= 0"
-        )
-
     return BasicIndexStrategy(
         shape,
         min_dims=min_dims,
