@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,26 +15,26 @@
 
 from collections import OrderedDict, abc
 from copy import copy
-from typing import Any, List, Sequence, Set, Union
+from typing import Any, List, Optional, Sequence, Set, Union
 
 import attr
 import numpy as np
 import pandas
 
-import hypothesis.extra.numpy as npst
-import hypothesis.internal.conjecture.utils as cu
-import hypothesis.strategies._internal.core as st
+from hypothesis import strategies as st
 from hypothesis.control import reject
 from hypothesis.errors import InvalidArgument
+from hypothesis.extra import numpy as npst
+from hypothesis.internal.conjecture import utils as cu
 from hypothesis.internal.coverage import check, check_function
-from hypothesis.internal.reflection import deprecated_posargs
 from hypothesis.internal.validation import (
     check_type,
     check_valid_interval,
     check_valid_size,
     try_convert,
 )
-from hypothesis.strategies._internal.strategies import Ex
+from hypothesis.strategies._internal.strategies import Ex, check_strategy
+from hypothesis.strategies._internal.utils import cacheable, defines_strategy
 
 try:
     from pandas.api.types import is_categorical_dtype
@@ -65,25 +65,21 @@ def elements_and_dtype(elements, dtype, source=None):
     if source is None:
         prefix = ""
     else:
-        prefix = "%s." % (source,)
+        prefix = f"{source}."
 
     if elements is not None:
-        st.check_strategy(elements, "%selements" % (prefix,))
+        check_strategy(elements, f"{prefix}elements")
     else:
         with check("dtype is not None"):
             if dtype is None:
                 raise InvalidArgument(
-                    (
-                        "At least one of %(prefix)selements or %(prefix)sdtype "
-                        "must be provided."
-                    )
-                    % {"prefix": prefix}
+                    f"At least one of {prefix}elements or {prefix}dtype must be provided."
                 )
 
     with check("is_categorical_dtype"):
         if is_categorical_dtype(dtype):
             raise InvalidArgument(
-                "%sdtype is categorical, which is currently unsupported" % (prefix,)
+                f"{prefix}dtype is categorical, which is currently unsupported"
             )
 
     dtype = try_convert(np.dtype, dtype, "dtype")
@@ -93,18 +89,18 @@ def elements_and_dtype(elements, dtype, source=None):
     elif dtype is not None:
 
         def convert_element(value):
-            name = "draw(%selements)" % (prefix,)
+            name = f"draw({prefix}elements)"
             try:
                 return np.array([value], dtype=dtype)[0]
             except TypeError:
                 raise InvalidArgument(
                     "Cannot convert %s=%r of type %s to dtype %s"
                     % (name, value, type(value).__name__, dtype.str)
-                )
+                ) from None
             except ValueError:
                 raise InvalidArgument(
-                    "Cannot convert %s=%r to type %s" % (name, value, dtype.str)
-                )
+                    f"Cannot convert {name}={value!r} to type {dtype.str}"
+                ) from None
 
         elements = elements.map(convert_element)
     assert elements is not None
@@ -151,10 +147,11 @@ class ValueIndexStrategy(st.SearchStrategy):
 DEFAULT_MAX_SIZE = 10
 
 
-@st.cacheable
-@st.defines_strategy
+@cacheable
+@defines_strategy()
 def range_indexes(
-    min_size: int = 0, max_size: int = None
+    min_size: int = 0,
+    max_size: Optional[int] = None,
 ) -> st.SearchStrategy[pandas.RangeIndex]:
     """Provides a strategy which generates an :class:`~pandas.Index` whose
     values are 0, 1, ..., n for some n.
@@ -173,16 +170,15 @@ def range_indexes(
     return st.integers(min_size, max_size).map(pandas.RangeIndex)
 
 
-@st.cacheable
-@st.defines_strategy
-@deprecated_posargs
+@cacheable
+@defines_strategy()
 def indexes(
     *,
-    elements: st.SearchStrategy[Ex] = None,
+    elements: Optional[st.SearchStrategy[Ex]] = None,
     dtype: Any = None,
     min_size: int = 0,
-    max_size: int = None,
-    unique: bool = True
+    max_size: Optional[int] = None,
+    unique: bool = True,
 ) -> st.SearchStrategy[pandas.Index]:
     """Provides a strategy for producing a :class:`pandas.Index`.
 
@@ -215,15 +211,14 @@ def indexes(
     return ValueIndexStrategy(elements, dtype, min_size, max_size, unique)
 
 
-@st.defines_strategy
-@deprecated_posargs
+@defines_strategy()
 def series(
     *,
-    elements: st.SearchStrategy[Ex] = None,
+    elements: Optional[st.SearchStrategy[Ex]] = None,
     dtype: Any = None,
-    index: st.SearchStrategy[Union[Sequence, pandas.Index]] = None,
-    fill: st.SearchStrategy[Ex] = None,
-    unique: bool = False
+    index: Optional[st.SearchStrategy[Union[Sequence, pandas.Index]]] = None,
+    fill: Optional[st.SearchStrategy[Ex]] = None,
+    unique: bool = False,
 ) -> st.SearchStrategy[pandas.Series]:
     """Provides a strategy for producing a :class:`pandas.Series`.
 
@@ -261,7 +256,7 @@ def series(
     if index is None:
         index = range_indexes()
     else:
-        st.check_strategy(index, "index")
+        check_strategy(index, "index")
 
     elements, dtype = elements_and_dtype(elements, dtype)
     index_strategy = index
@@ -332,14 +327,13 @@ class column:
     unique = attr.ib(default=False)
 
 
-@deprecated_posargs
 def columns(
     names_or_number: Union[int, Sequence[str]],
     *,
     dtype: Any = None,
-    elements: st.SearchStrategy[Ex] = None,
-    fill: st.SearchStrategy[Ex] = None,
-    unique: bool = False
+    elements: Optional[st.SearchStrategy[Ex]] = None,
+    fill: Optional[st.SearchStrategy[Ex]] = None,
+    unique: bool = False,
 ) -> List[column]:
     """A convenience function for producing a list of :class:`column` objects
     of the same general shape.
@@ -351,7 +345,7 @@ def columns(
     create the columns.
     """
     if isinstance(names_or_number, (int, float)):
-        names = [None] * names_or_number  # type: list
+        names: List[Union[int, str, None]] = [None] * names_or_number
     else:
         names = list(names_or_number)
     return [
@@ -360,13 +354,12 @@ def columns(
     ]
 
 
-@deprecated_posargs
-@st.defines_strategy
+@defines_strategy()
 def data_frames(
-    columns: Sequence[column] = None,
+    columns: Optional[Sequence[column]] = None,
     *,
-    rows: st.SearchStrategy[Union[dict, Sequence[Any]]] = None,
-    index: st.SearchStrategy[Ex] = None
+    rows: Optional[st.SearchStrategy[Union[dict, Sequence[Any]]]] = None,
+    index: Optional[st.SearchStrategy[Ex]] = None,
 ) -> st.SearchStrategy[pandas.DataFrame]:
     """Provides a strategy for producing a :class:`pandas.DataFrame`.
 
@@ -473,7 +466,7 @@ def data_frames(
     if index is None:
         index = range_indexes()
     else:
-        st.check_strategy(index, "index")
+        check_strategy(index, "index")
 
     index_strategy = index
 
@@ -504,17 +497,17 @@ def data_frames(
             return rows_only()
 
     assert columns is not None
-    cols = try_convert(tuple, columns, "columns")  # type: Sequence[column]
+    cols = try_convert(tuple, columns, "columns")
 
     rewritten_columns = []
-    column_names = set()  # type: Set[str]
+    column_names: Set[str] = set()
 
     for i, c in enumerate(cols):
-        check_type(column, c, "columns[%d]" % (i,))
+        check_type(column, c, f"columns[{i}]")
 
         c = copy(c)
         if c.name is None:
-            label = "columns[%d]" % (i,)
+            label = f"columns[{i}]"
             c.name = i
         else:
             label = c.name
@@ -525,10 +518,10 @@ def data_frames(
                     "Column names must be hashable, but columns[%d].name was "
                     "%r of type %s, which cannot be hashed."
                     % (i, c.name, type(c.name).__name__)
-                )
+                ) from None
 
         if c.name in column_names:
-            raise InvalidArgument("duplicate definition of column name %r" % (c.name,))
+            raise InvalidArgument(f"duplicate definition of column name {c.name!r}")
 
         column_names.add(c.name)
 
@@ -536,8 +529,7 @@ def data_frames(
 
         if c.dtype is None and rows is not None:
             raise InvalidArgument(
-                "Must specify a dtype for all columns when combining rows with"
-                " columns."
+                "Must specify a dtype for all columns when combining rows with columns."
             )
 
         c.fill = npst.fill_for(
@@ -647,6 +639,11 @@ def data_frames(
                                 try:
                                     as_list[i] = fills[i]
                                 except KeyError:
+                                    if c.fill.is_empty:
+                                        raise InvalidArgument(
+                                            f"Empty fill strategy in {c!r} cannot "
+                                            f"complete row {original_row!r}"
+                                        ) from None
                                     fills[i] = draw(c.fill)
                                     as_list[i] = fills[i]
                         for k in row:
@@ -671,14 +668,17 @@ def data_frames(
 
                     if len(row) > len(rewritten_columns):
                         raise InvalidArgument(
-                            (
-                                "Row %r contains too many entries. Has %d but "
-                                "expected at most %d"
-                            )
-                            % (original_row, len(row), len(rewritten_columns))
+                            f"Row {original_row!r} contains too many entries. Has "
+                            f"{len(row)} but expected at most {len(rewritten_columns)}"
                         )
                     while len(row) < len(rewritten_columns):
-                        row.append(draw(rewritten_columns[len(row)].fill))
+                        c = rewritten_columns[len(row)]
+                        if c.fill.is_empty:
+                            raise InvalidArgument(
+                                f"Empty fill strategy in {c!r} cannot "
+                                f"complete row {original_row!r}"
+                            )
+                        row.append(draw(c.fill))
                     result.iloc[row_index] = row
                     break
                 else:
