@@ -112,6 +112,7 @@ from hypothesis.strategies._internal.strategies import (
 from hypothesis.strategies._internal.strings import (
     FixedSizeBytes,
     OneCharStringStrategy,
+    TextStrategy,
 )
 from hypothesis.strategies._internal.utils import cacheable, defines_strategy
 from hypothesis.utils.conventions import InferType, infer, not_set
@@ -643,7 +644,7 @@ def text(
         )
     if (max_size == 0 or char_strategy.is_empty) and not min_size:
         return just("")
-    return lists(char_strategy, min_size=min_size, max_size=max_size).map("".join)
+    return TextStrategy(char_strategy, min_size=min_size, max_size=max_size)
 
 
 @cacheable
@@ -814,6 +815,12 @@ class BuildsStrategy(SearchStrategy):
         tuples(*self.args).validate()
         fixed_dictionaries(self.kwargs).validate()
 
+    def __repr__(self):
+        bits = [get_pretty_function_description(self.target)]
+        bits.extend(map(repr, self.args))
+        bits.extend(f"{k}={v!r}" for k, v in self.kwargs.items())
+        return f"builds({', '.join(bits)})"
+
 
 # The ideal signature builds(target, /, *args, **kwargs) is unfortunately a
 # SyntaxError before Python 3.8 so we emulate it with manual argument unpacking.
@@ -875,16 +882,16 @@ def builds(
         # Otherwise, try using type hints
         hints = get_type_hints(target)
         if to_infer - set(hints):
+            badargs = ", ".join(sorted(to_infer - set(hints)))
             raise InvalidArgument(
-                "passed infer for %s, but there is no type annotation"
-                % (", ".join(sorted(to_infer - set(hints))))
+                f"passed infer for {badargs}, but there is no type annotation"
             )
         for kw in set(hints) & (required | to_infer):
             kwargs[kw] = from_type(hints[kw])
     return BuildsStrategy(target, args, kwargs)
 
 
-if sys.version_info[:2] >= (3, 8):  # pragma: no cover
+if sys.version_info[:2] >= (3, 8):  # pragma: no branch
     # See notes above definition - this signature is compatible and better
     # matches the semantics of the function.  Great for documentation!
     sig = signature(builds)
@@ -1626,11 +1633,8 @@ def uuids(*, version: Optional[int] = None) -> SearchStrategy[UUID]:
     """
     if version not in (None, 1, 2, 3, 4, 5):
         raise InvalidArgument(
-            (
-                "version=%r, but version must be in (None, 1, 2, 3, 4, 5) "
-                "to pass to the uuid.UUID constructor."
-            )
-            % (version,)
+            f"version={version!r}, but version must be in "
+            "(None, 1, 2, 3, 4, 5) to pass to the uuid.UUID constructor."
         )
     return shared(
         randoms(use_true_random=True), key="hypothesis.strategies.uuids.generator"
