@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -13,6 +13,7 @@
 #
 # END HEADER
 
+import builtins
 import collections
 import datetime
 import decimal
@@ -23,6 +24,7 @@ import io
 import ipaddress
 import numbers
 import os
+import re
 import sys
 import typing
 import uuid
@@ -31,7 +33,9 @@ from types import FunctionType
 
 from hypothesis import strategies as st
 from hypothesis.errors import InvalidArgument, ResolutionFailed
-from hypothesis.internal.compat import ForwardRef, typing_root_type
+from hypothesis.internal.compat import PYPY, ForwardRef, typing_root_type
+from hypothesis.internal.conjecture.utils import many as conjecture_utils_many
+from hypothesis.strategies._internal.datetime import zoneinfo  # type: ignore
 from hypothesis.strategies._internal.ipaddress import (
     SPECIAL_IPv4_RANGES,
     SPECIAL_IPv6_RANGES,
@@ -55,11 +59,26 @@ try:
 except ImportError:
     _GenericAlias = ()
 
+try:
+    from typing import _AnnotatedAlias  # type: ignore
+except ImportError:
+    try:
+        from typing_extensions import _AnnotatedAlias
+    except ImportError:
+        try:
+            from typing_extensions import (  # type: ignore
+                AnnotatedMeta as _AnnotatedAlias,
+            )
+
+            assert sys.version_info[:2] == (3, 6)
+        except ImportError:
+            _AnnotatedAlias = ()
+
 
 def type_sorting_key(t):
     """Minimise to None, then non-container types, then container types."""
-    if not is_a_type(t):
-        raise InvalidArgument("thing=%s must be a type" % (t,))
+    if not (is_a_type(t) or is_typing_literal(t)):  # This branch is for Python < 3.8
+        raise InvalidArgument(f"thing={t} must be a type")  # pragma: no cover
     if t is None or t is type(None):  # noqa: E721
         return (-1, repr(t))
     if not isinstance(t, type):  # pragma: no cover
@@ -68,30 +87,63 @@ def type_sorting_key(t):
     return (int(issubclass(t, collections.abc.Container)), repr(t))
 
 
+def _compatible_args(args, superclass_args):
+    """Check that the args of two generic types are compatible for try_issubclass."""
+    assert superclass_args is not None
+    if args is None:
+        return True
+    return len(args) == len(superclass_args) and all(
+        # "a==b or either is a typevar" is a hacky approximation, but it's
+        # good enough for all the cases that I've seen so far and has the
+        # substantial virtue of (relative) simplicity.
+        a == b or isinstance(a, typing.TypeVar) or isinstance(b, typing.TypeVar)
+        for a, b in zip(args, superclass_args)
+    )
+
+
 def try_issubclass(thing, superclass):
-    thing = getattr(thing, "__origin__", None) or thing
-    superclass = getattr(superclass, "__origin__", None) or superclass
     try:
-        return issubclass(thing, superclass)
-    except (AttributeError, TypeError):  # pragma: no cover
-        # Some types can't be the subject or object of an instance or
-        # subclass check under Python 3.5
+        # In this case we're looking at two distinct classes - which might be generics.
+        # That brings in some complications:
+        if issubclass(
+            getattr(thing, "__origin__", None) or thing,
+            getattr(superclass, "__origin__", None) or superclass,
+        ):
+            superclass_args = getattr(superclass, "__args__", None)
+            if sys.version_info[:2] == (3, 6) or not superclass_args:
+                # Python 3.6 doesn't have PEP-560 semantics or __orig_bases__,
+                # so there's no point continuing; we therefore stop early.
+                # The superclass is not generic, so we're definitely a subclass.
+                return True
+            # Sadly this is just some really fiddly logic to handle all the cases
+            # of user-defined generic types, types inheriting from parametrised
+            # generics, and so on.  If you need to change this code, read PEP-560
+            # and Hypothesis issue #2951 closely first, and good luck.  The tests
+            # will help you, I hope - good luck.
+            if getattr(thing, "__args__", None) is not None:
+                return True
+            for orig_base in getattr(thing, "__orig_bases__", None) or [None]:
+                args = getattr(orig_base, "__args__", None)
+                if _compatible_args(args, superclass_args):
+                    return True
+        return False
+    except (AttributeError, TypeError):
+        # Some types can't be the subject or object of an instance or subclass check
         return False
 
 
 def is_a_new_type(thing):
-    # At runtime, `typing.NewType` returns an identity function rather
-    # than an actual type, but we can check whether that thing matches.
-    return (
-        hasattr(thing, "__supertype__")
-        and (
-            hasattr(typing, "NewType")
-            and getattr(thing, "__module__", None) == "typing"
-            or hasattr(typing_extensions, "NewType")
-            and getattr(thing, "__module__", None) == "typing_extensions"
+    if not isinstance(typing.NewType, type):
+        # At runtime, `typing.NewType` returns an identity function rather
+        # than an actual type, but we can check whether that thing matches.
+        return (
+            hasattr(thing, "__supertype__")
+            and getattr(thing, "__module__", None) in ("typing", "typing_extensions")
+            and inspect.isfunction(thing)
         )
-        and inspect.isfunction(thing)
-    )
+    # In 3.10 and later, NewType is actually a class - which simplifies things.
+    # See https://bugs.python.org/issue44353 for links to the various patches.
+    return isinstance(thing, typing.NewType)  # pragma: no cover  # on 3.8, anyway
 
 
 def is_a_type(thing):
@@ -104,10 +156,36 @@ def is_typing_literal(thing):
         hasattr(typing, "Literal")
         and getattr(thing, "__origin__", None) == typing.Literal
         or hasattr(typing_extensions, "Literal")
-        and getattr(thing, "__origin__", None) == typing_extensions.Literal
+        and (
+            getattr(thing, "__origin__", None) == typing_extensions.Literal
+            or sys.version_info[:2] == (3, 6)
+            and isinstance(thing, type(typing_extensions.Literal[None]))
+        )
     )
 
 
+def is_annotated_type(thing):
+    return (
+        isinstance(thing, _AnnotatedAlias)
+        and getattr(thing, "__args__", None) is not None
+    )
+
+
+def find_annotated_strategy(annotated_type):  # pragma: no cover
+    flattened_meta = []
+
+    all_args = (
+        *getattr(annotated_type, "__args__", ()),
+        *getattr(annotated_type, "__metadata__", ()),
+    )
+    for arg in all_args:
+        if is_annotated_type(arg):
+            flattened_meta.append(find_annotated_strategy(arg))
+        if isinstance(arg, st.SearchStrategy):
+            flattened_meta.append(arg)
+    return flattened_meta[-1] if flattened_meta else None
+
+
 def has_type_arguments(type_):
     """Decides whethere or not this type has applied type arguments."""
     args = getattr(type_, "__args__", None)
@@ -121,9 +199,9 @@ def has_type_arguments(type_):
 
 
 def is_generic_type(type_):
-    """Decides whethere a given type is generic or not."""
+    """Decides whether a given type is generic or not."""
     # The ugly truth is that `MyClass`, `MyClass[T]`, and `MyClass[int]` are very different.
-    # In different python versions the might have the same type (3.6)
+    # In different python versions they might have the same type (3.6)
     # or it can be regular type vs `_GenericAlias` (3.7+)
     # We check for `MyClass[T]` and `MyClass[int]` with the first condition,
     # while the second condition is for `MyClass` in `python3.7+`.
@@ -148,9 +226,8 @@ def _try_import_forward_ref(thing, bound):  # pragma: no cover
             and getattr(thing, "__module__", None) == "typing"
         ):
             raise ResolutionFailed(
-                "Looks like you are using python3.6 or any of the previous versions "
-                "together with a TypeVar bound to a ForwardRef. "
-                "It is not supported. Upgrading to python3.7+ will solve this problem."
+                "It looks like you're using a TypeVar bound to a ForwardRef on Python "
+                "3.6, which is not supported - try ugrading to Python 3.7 or later."
             ) from None
         # We fallback to `ForwardRef` instance, you can register it as a type as well:
         # >>> from typing import ForwardRef
@@ -168,15 +245,6 @@ def from_typing_type(thing):
     # information to sensibly resolve to strategies at runtime.
     # Finally, we run a variation of the subclass lookup in `st.from_type`
     # among generic types in the lookup.
-    #
-    # Under 3.6 Union is handled directly in st.from_type, as the argument is
-    # not an instance of `type`. However, under Python 3.5 Union *is* a type
-    # and we have to handle it here, including failing if it has no parameters.
-    if hasattr(thing, "__union_params__"):  # pragma: no cover
-        args = sorted(thing.__union_params__ or (), key=type_sorting_key)
-        if not args:
-            raise ResolutionFailed("Cannot resolve Union of no types.")
-        return st.one_of([st.from_type(t) for t in args])
     if getattr(thing, "__origin__", None) == tuple or isinstance(
         thing, getattr(typing, "TupleMeta", ())
     ):
@@ -191,11 +259,9 @@ def from_typing_type(thing):
         elif len(elem_types) == 1 and elem_types[0] == ():
             return st.tuples()  # Empty tuple; see issue #1583
         return st.tuples(*map(st.from_type, elem_types))
-    if (
-        hasattr(typing, "Final") and getattr(thing, "__origin__", None) == typing.Final
-    ):  # pragma: no cover  # new in Python 3.8
+    if hasattr(typing, "Final") and getattr(thing, "__origin__", None) == typing.Final:
         return st.one_of([st.from_type(t) for t in thing.__args__])
-    if is_typing_literal(thing):  # pragma: no cover  # new in Python 3.8
+    if is_typing_literal(thing):
         args_dfs_stack = list(thing.__args__)
         literals = []
         while args_dfs_stack:
@@ -205,9 +271,20 @@ def from_typing_type(thing):
             else:
                 literals.append(arg)
         return st.sampled_from(literals)
+    if is_annotated_type(thing):  # pragma: no cover
+        # This requires Python 3.9+ or the typing_extensions package
+        annotated_strategy = find_annotated_strategy(thing)
+        if annotated_strategy is not None:
+            return annotated_strategy
+        args = thing.__args__
+        assert args, "it's impossible to make an annotated type with no args"
+        annotated_type = args[0]
+        return st.from_type(annotated_type)
     # Now, confirm that we're dealing with a generic type as we expected
-    if not isinstance(thing, typing_root_type):  # pragma: no cover
-        raise ResolutionFailed("Cannot resolve %s to a strategy" % (thing,))
+    if sys.version_info[:2] < (3, 9) and not isinstance(
+        thing, typing_root_type
+    ):  # pragma: no cover
+        raise ResolutionFailed(f"Cannot resolve {thing} to a strategy")
 
     # Some "generic" classes are not generic *in* anything - for example both
     # Hashable and Sized have `__args__ == ()` on Python 3.7 or later.
@@ -217,7 +294,7 @@ def from_typing_type(thing):
         typing.Hashable is not collections.abc.Hashable
         and origin in vars(collections.abc).values()
         and len(getattr(thing, "__args__", None) or []) == 0
-    ):  # pragma: no cover  # impossible on 3.6 where we measure coverage.
+    ):
         return st.from_type(origin)
 
     # Parametrised generic types have their __origin__ attribute set to the
@@ -228,14 +305,17 @@ def from_typing_type(thing):
         for k, v in _global_type_lookup.items()
         if is_generic_type(k) and try_issubclass(k, thing)
     }
-    if typing.Dict in mapping:
-        # The subtype relationships between generic and concrete View types
-        # are sometimes inconsistent under Python 3.5, so we pop them out to
-        # preserve our invariant that all examples of from_type(T) are
-        # instances of type T - and simplify the strategy for abstract types
-        # such as Container
-        for t in (typing.KeysView, typing.ValuesView, typing.ItemsView):
-            mapping.pop(t, None)
+    if typing.Dict in mapping or typing.Set in mapping:
+        # ItemsView can cause test_lookup.py::test_specialised_collection_types
+        # to fail, due to weird isinstance behaviour around the elements.
+        mapping.pop(typing.ItemsView, None)
+        if sys.version_info[:2] == (3, 6):  # pragma: no cover
+            # `isinstance(dict().values(), Container) is False` on py36 only -_-
+            mapping.pop(typing.ValuesView, None)
+    if typing.Deque in mapping and len(mapping) > 1:
+        # Resolving generic sequences to include a deque is more trouble for e.g.
+        # the ghostwriter than it's worth, via undefined names in the repr.
+        mapping.pop(typing.Deque)
     if len(mapping) > 1:
         # issubclass treats bytestring as a kind of sequence, which it is,
         # but treating it as such breaks everything else when it is presumed
@@ -249,8 +329,6 @@ def from_typing_type(thing):
         elem_type = (getattr(thing, "__args__", None) or ["not int"])[0]
         if getattr(elem_type, "__origin__", None) is typing.Union:
             union_elems = elem_type.__args__
-        elif hasattr(elem_type, "__union_params__"):  # pragma: no cover
-            union_elems = elem_type.__union_params__  # python 3.5 only
         else:
             union_elems = ()
         if not any(
@@ -258,6 +336,12 @@ def from_typing_type(thing):
             for T in list(union_elems) + [elem_type]
         ):
             mapping.pop(typing.ByteString, None)
+    elif (
+        (not mapping)
+        and isinstance(thing, ForwardRef)
+        and thing.__forward_arg__ in vars(builtins)
+    ):
+        return st.from_type(getattr(builtins, thing.__forward_arg__))
     strategies = [
         v if isinstance(v, st.SearchStrategy) else v(thing)
         for k, v in mapping.items()
@@ -266,8 +350,8 @@ def from_typing_type(thing):
     empty = ", ".join(repr(s) for s in strategies if s.is_empty)
     if empty or not strategies:
         raise ResolutionFailed(
-            "Could not resolve %s to a strategy; consider using "
-            "register_type_strategy" % (empty or thing,)
+            f"Could not resolve {empty or thing} to a strategy; "
+            "consider using register_type_strategy"
         )
     return st.one_of(strategies)
 
@@ -299,7 +383,9 @@ utc_offsets = st.builds(
 # As a general rule, we try to limit this to scalars because from_type()
 # would have to decide on arbitrary collection elements, and we'd rather
 # not (with typing module generic types and some builtins as exceptions).
-_global_type_lookup = {
+_global_type_lookup: typing.Dict[
+    type, typing.Union[st.SearchStrategy, typing.Callable[[type], st.SearchStrategy]]
+] = {
     type(None): st.none(),
     bool: st.booleans(),
     int: st.integers(),
@@ -339,7 +425,7 @@ _global_type_lookup = {
         st.none() | st.integers(),
     ),
     range: st.one_of(
-        st.integers(min_value=0).map(range),
+        st.builds(range, st.integers(min_value=0)),
         st.builds(range, st.integers(), st.integers()),
         st.builds(range, st.integers(), st.integers(), st.integers().filter(bool)),
     ),
@@ -355,15 +441,49 @@ _global_type_lookup = {
         _networks(128).map(lambda x: ipaddress.IPv6Network(x, strict=False)),
         st.sampled_from(SPECIAL_IPv6_RANGES).map(ipaddress.IPv6Network),
     ),
+    os.PathLike: st.builds(PurePath, st.text()),
+    UnicodeDecodeError: st.builds(
+        UnicodeDecodeError,
+        st.just("unknown encoding"),
+        st.just(b""),
+        st.just(0),
+        st.just(0),
+        st.just("reason"),
+    ),
+    UnicodeEncodeError: st.builds(
+        UnicodeEncodeError,
+        st.just("unknown encoding"),
+        st.text(),
+        st.just(0),
+        st.just(0),
+        st.just("reason"),
+    ),
+    UnicodeTranslateError: st.builds(
+        UnicodeTranslateError, st.text(), st.just(0), st.just(0), st.just("reason")
+    ),
+    enumerate: st.builds(enumerate, st.just(())),
+    filter: st.builds(filter, st.just(lambda _: None), st.just(())),
+    map: st.builds(map, st.just(lambda _: None), st.just(())),
+    reversed: st.builds(reversed, st.just(())),
+    classmethod: st.builds(classmethod, st.just(lambda self: self)),
+    staticmethod: st.builds(staticmethod, st.just(lambda self: self)),
+    super: st.builds(super, st.from_type(type)),
     # Pull requests with more types welcome!
 }
+if zoneinfo is not None:  # pragma: no branch
+    _global_type_lookup[zoneinfo.ZoneInfo] = st.timezones()
+if PYPY:
+    _global_type_lookup[builtins.sequenceiterator] = st.builds(iter, st.tuples())  # type: ignore
 
 _global_type_lookup[type] = st.sampled_from(
     [type(None)] + sorted(_global_type_lookup, key=str)
 )
 
-if sys.version_info[:2] >= (3, 6):  # pragma: no branch
-    _global_type_lookup[os.PathLike] = st.builds(PurePath, st.text())
+if sys.version_info[:2] >= (3, 7):  # pragma: no branch
+    _global_type_lookup[re.Match] = (
+        st.text().map(lambda c: re.match(".", c, flags=re.DOTALL)).filter(bool)
+    )
+    _global_type_lookup[re.Pattern] = st.builds(re.compile, st.sampled_from(["", b""]))
 if sys.version_info[:2] >= (3, 9):  # pragma: no cover
     # subclass of MutableMapping, and in Python 3.9 we resolve to a union
     # which includes this... but we don't actually ever want to build one.
@@ -383,10 +503,11 @@ except ImportError:
 
 _global_type_lookup.update(
     {
-        typing.ByteString: st.binary() | st.binary().map(bytearray),
-        # Reversible is somehow a subclass of Hashable, so we tuplize it.
-        # See also the discussion at https://bugs.python.org/issue39046
-        typing.Reversible: st.lists(st.integers()).map(tuple),
+        # Note: while ByteString notionally also represents the bytearray and
+        # memoryview types, it is a subclass of Hashable and those types are not.
+        # We therefore only generate the bytes type.
+        typing.ByteString: st.binary(),
+        collections.abc.ByteString: st.binary(),
         # TODO: SupportsAbs and SupportsRound should be covariant, ie have functions.
         typing.SupportsAbs: st.one_of(
             st.booleans(),
@@ -425,9 +546,7 @@ _global_type_lookup.update(
             st.uuids(),
             st.decimals(),
             # this generates strings that should able to be parsed into integers
-            st.from_regex(r"-?\d+", fullmatch=True).filter(
-                lambda value: can_cast(int, value)
-            ),
+            st.from_regex(r"\A-?\d+\Z").filter(functools.partial(can_cast, int)),
         ),
         typing.SupportsBytes: st.one_of(
             st.booleans(),
@@ -436,13 +555,11 @@ _global_type_lookup.update(
             # As with Reversible, we tuplize this for compatibility with Hashable.
             st.lists(st.integers(0, 255)).map(tuple),  # type: ignore
         ),
-        # xIO are only available in .io on Python 3.5, but available directly
-        # as typing.*IO from 3.6 onwards and mypy 0.730 errors on the compat form.
-        typing.io.BinaryIO: st.builds(io.BytesIO, st.binary()),  # type: ignore
-        typing.io.TextIO: st.builds(io.StringIO, st.text()),  # type: ignore
+        typing.BinaryIO: st.builds(io.BytesIO, st.binary()),
+        typing.TextIO: st.builds(io.StringIO, st.text()),
     }
 )
-if hasattr(typing, "SupportsIndex"):  # pragma: no cover  # new in Python 3.8
+if hasattr(typing, "SupportsIndex"):  # pragma: no branch  # new in Python 3.8
     _global_type_lookup[typing.SupportsIndex] = st.integers() | st.booleans()  # type: ignore
 
 
@@ -461,8 +578,9 @@ def register(type_, fallback=None, *, module=typing):
 
         @functools.wraps(func)
         def really_inner(thing):
+            # This branch is for Python < 3.8, when __args__ was not always tracked
             if getattr(thing, "__args__", None) is None:
-                return fallback
+                return fallback  # pragma: no cover
             return func(thing)
 
         _global_type_lookup[type_] = really_inner
@@ -471,25 +589,26 @@ def register(type_, fallback=None, *, module=typing):
     return inner
 
 
-@register("Type")
+@register(typing.Type)
 @register("Type", module=typing_extensions)
 def resolve_Type(thing):
     if getattr(thing, "__args__", None) is None:
-        return st.just(type)
+        # This branch is for Python < 3.8, when __args__ was not always tracked
+        return st.just(type)  # pragma: no cover
     args = (thing.__args__[0],)
     if getattr(args[0], "__origin__", None) is typing.Union:
         args = args[0].__args__
-    elif hasattr(args[0], "__union_params__"):  # pragma: no cover
-        args = args[0].__union_params__
-    if isinstance(ForwardRef, type):  # pragma: no cover
-        # Duplicate check from from_type here - only paying when needed.
-        for a in args:
-            if type(a) == ForwardRef:
+    # Duplicate check from from_type here - only paying when needed.
+    args = list(args)
+    for i, a in enumerate(args):
+        if type(a) == ForwardRef:
+            try:
+                args[i] = getattr(builtins, a.__forward_arg__)
+            except AttributeError:
                 raise ResolutionFailed(
-                    "thing=%s cannot be resolved.  Upgrading to "
-                    "python>=3.6 may fix this problem via improvements "
-                    "to the typing module." % (thing,)
-                )
+                    f"Cannot find the type referenced by {thing} - try using "
+                    f"st.register_type_strategy({thing}, st.from_type(...))"
+                ) from None
     return st.sampled_from(sorted(args, key=type_sorting_key))
 
 
@@ -541,7 +660,7 @@ def resolve_Dict(thing):
     )
 
 
-@register("DefaultDict", st.builds(collections.defaultdict))
+@register(typing.DefaultDict, st.builds(collections.defaultdict))
 @register("DefaultDict", st.builds(collections.defaultdict), module=typing_extensions)
 def resolve_DefaultDict(thing):
     return resolve_Dict(thing).map(lambda d: collections.defaultdict(None, d))
@@ -571,6 +690,74 @@ def resolve_Iterator(thing):
     return st.iterables(st.from_type(thing.__args__[0]))
 
 
+@register(typing.Counter, st.builds(collections.Counter))
+def resolve_Counter(thing):
+    return st.dictionaries(
+        keys=st.from_type(thing.__args__[0]),
+        values=st.integers(),
+    ).map(collections.Counter)
+
+
+@register(typing.Deque, st.builds(collections.deque))
+def resolve_deque(thing):
+    return st.lists(st.from_type(thing.__args__[0])).map(collections.deque)
+
+
+@register(typing.ChainMap, st.builds(dict).map(collections.ChainMap))
+def resolve_ChainMap(thing):
+    return resolve_Dict(thing).map(collections.ChainMap)
+
+
+@register("OrderedDict", st.builds(dict).map(collections.OrderedDict))
+def resolve_OrderedDict(thing):
+    # typing.OrderedDict is new in Python 3.7.2
+    return resolve_Dict(thing).map(collections.OrderedDict)
+
+
+@register(typing.Pattern, st.builds(re.compile, st.sampled_from(["", b""])))
+def resolve_Pattern(thing):
+    if isinstance(thing.__args__[0], typing.TypeVar):
+        return st.builds(re.compile, st.sampled_from(["", b""]))
+    return st.just(re.compile(thing.__args__[0]()))
+
+
+@register(  # pragma: no branch  # coverage does not see lambda->exit branch
+    typing.Match,
+    st.text().map(lambda c: re.match(".", c, flags=re.DOTALL)).filter(bool),
+)
+def resolve_Match(thing):
+    if thing.__args__[0] == bytes:
+        return (
+            st.binary(min_size=1)
+            .map(lambda c: re.match(b".", c, flags=re.DOTALL))
+            .filter(bool)
+        )
+    return st.text().map(lambda c: re.match(".", c, flags=re.DOTALL)).filter(bool)
+
+
+class GeneratorStrategy(st.SearchStrategy):
+    def __init__(self, yields, returns):
+        assert isinstance(yields, st.SearchStrategy)
+        assert isinstance(returns, st.SearchStrategy)
+        self.yields = yields
+        self.returns = returns
+
+    def __repr__(self):
+        return f"<generators yields={self.yields!r} returns={self.returns!r}>"
+
+    def do_draw(self, data):
+        elements = conjecture_utils_many(data, min_size=0, max_size=100, average_size=5)
+        while elements.more():
+            yield data.draw(self.yields)
+        return data.draw(self.returns)
+
+
+@register(typing.Generator, GeneratorStrategy(st.none(), st.none()))
+def resolve_Generator(thing):
+    yields, _, returns = thing.__args__
+    return GeneratorStrategy(st.from_type(yields), st.from_type(returns))
+
+
 @register(typing.Callable, st.functions())
 def resolve_Callable(thing):
     # Generated functions either accept no arguments, or arbitrary arguments.
@@ -578,15 +765,19 @@ def resolve_Callable(thing):
     # use of keyword arguments and we'd rather not force positional-only.
     if not thing.__args__:  # pragma: no cover  # varies by minor version
         return st.functions()
+    # Note that a list can only appear in __args__ under Python 3.9 with the
+    # collections.abc version; see https://bugs.python.org/issue42195
     return st.functions(
-        like=(lambda: None) if len(thing.__args__) == 1 else (lambda *a, **k: None),
+        like=(lambda: None)
+        if len(thing.__args__) == 1 or thing.__args__[0] == []
+        else (lambda *a, **k: None),
         returns=st.from_type(thing.__args__[-1]),
     )
 
 
 @register(typing.TypeVar)
 def resolve_TypeVar(thing):
-    type_var_key = "typevar=%r" % (thing,)
+    type_var_key = f"typevar={thing!r}"
 
     if getattr(thing, "__bound__", None) is not None:
         bound = thing.__bound__
@@ -601,7 +792,7 @@ def resolve_TypeVar(thing):
         # each part of the union, wraps that in shared so that we only generate
         # from one type per testcase, and flatmaps that back to instances.
         return st.shared(
-            st.sampled_from(strat.original_strategies), key=type_var_key,
+            st.sampled_from(strat.original_strategies), key=type_var_key
         ).flatmap(lambda s: s)
 
     builtin_scalar_types = [type(None), bool, int, float, str, bytes]
